-----------------------------------------------
-- Company          :    tud
-- Author           :    schreiter
-- E-Mail           :    <email>
--
-- Filename         :   euler_backward.vhd
-- Project Name     :   OSM
-- Subproject Name  :   s_vhdl
-- Description      :   <short description>
--
-- Create Date      :   Fri Nov 18 08:26:56 2016
-- Last Change      :   $Date$
-- by               :   $Author$
-----------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

--SYNOPSYS translate_off

--Uncomment the following two lines to use the declarations that are 
--provided for instantiating Xilinx primitive components
--library UNISIM;
--use UNISIM.VComponents.all;

--SYNOPSYS translate_on

entity euler_backward is
    generic (
      delta_t : time := 1 ns);          -- Euler's method constant step size
    port ( u_i : in real;
           u_o : out real);
end entity euler_backward;

architecture beh_vhdl of euler_backward is
  constant R : real := 8.0e6;           -- both resistors
  constant C : real := 1.0e-12;         -- both capacitors
  
  constant k : real := real(delta_t / ps) * 1.0e-12 / (R*C) ;                  -- delta_t / (R*C)

  signal u1 : real := 0.0;              -- voltage on C1
  signal u2 : real := 0.0;              -- voltage on C2
begin
            
  process
    variable u1_new, u2_new : real;
  begin
    wait for delta_t;
    u1_new := (k*u_i + u1 + k/(1.0+k)*u2) / (1.0+2.0*k-k*k/(1.0+k));
    u2_new := (k*u1_new + u2) / (1.0+k);
    u1 <= u1_new;
    u2 <= u2_new;
  end process;
  
  u_o <= u2;
  
end architecture beh_vhdl;
