-----------------------------------------------
-- Company          :    tud
-- Author           :    schreiter
-- E-Mail           :    <email>
--
-- Filename         :   euler_forward.vhd
-- Project Name     :   OSM
-- Subproject Name  :   s_vhdl
-- Description      :   <short description>
--
-- Create Date      :   Fri Nov 18 08:01:51 2016
-- Last Change      :   $Date$
-- by               :   $Author$
-----------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

--SYNOPSYS translate_off

--Uncomment the following two lines to use the declarations that are 
--provided for instantiating Xilinx primitive components
--library UNISIM;
--use UNISIM.VComponents.all;

--SYNOPSYS translate_on

entity euler_forward is
  generic (
    delta_t : time := 1 ns);            -- Euler's method constant step size
    port ( u_i : in real;
           u_o : out real);
end entity euler_forward;

architecture beh_vhdl of euler_forward is
  constant R : real := 8.0e6;           -- both resistors
  constant C : real := 1.0e-12;         -- both capacitors

  -- constant omega : real := 1.0/(R*C);
    constant k : real := real(delta_t / ps) * 1.0e-12 / (R*C) ;                  -- delta_t / (R*C)

  signal u1 : real := 0.0;              -- voltage on C1
  signal u2 : real := 0.0;              -- voltage on C2
begin

  process
  begin
    wait for delta_t;
    u1 <= u1 + (u_i - u1 + u2 - u1) * k; -- real( delta_t / ps)*1.0e-12 * omega;
    u2 <= u2 + (u1 - u2) * k; -- real( delta_t / ps)*1.0e-12 * omega;
  end process;
  
  u_o <= u2;
  
end architecture beh_vhdl;
