-----------------------------------------------
-- Company          :    tud
-- Author           :    schreiter
-- E-Mail           :    <email>
--
-- Filename         :   rc_lowpass.vhd
-- Project Name     :   OSM
-- Subproject Name  :   s_vhdl
-- Description      :   <short description>
--
-- Create Date      :   Fri Nov 18 08:00:33 2016
-- Last Change      :   $Date$
-- by               :   $Author$
-----------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use IEEE.math_real.all;

--SYNOPSYS translate_off

--Uncomment the following two lines to use the declarations that are 
--provided for instantiating Xilinx primitive components
--library UNISIM;
--use UNISIM.VComponents.all;

--SYNOPSYS translate_on

entity rc_lowpass is
end entity rc_lowpass;

architecture rtl_vhdl of rc_lowpass is

  component euler_forward is
    generic (
      delta_t : time);
    port ( u_i : in real;
           u_o : out real);
  end component euler_forward;

  component euler_backward is
    generic (
      delta_t : time);
    port ( u_i : in real;
           u_o : out real);
  end component euler_backward;

  component explicit is
    port ( u_i : in real;
           u_o : out real);
  end component explicit;

  signal u_i : real := 0.0;
  signal u_o_euler_fwd, u_o_euler_rev, u_o_explicit : real;

  signal pay_load : real := 0.0;               -- payload signal

  constant f_signal : real := 5.0e3;    -- 5 kHz payload signal
  constant a_signal : real := 1.0;      -- 1 Volt signal amplitude
  constant t_pwm : time := 1 us;        -- 1 MHz PWM frequency
  
begin

  i_euler_forward : euler_forward
    generic map (
      delta_t => t_pwm / 100)
    port map (
      u_i => u_i,
      u_o => u_o_euler_fwd);
  
  
  i_euler_backward : euler_backward
    generic map (
      delta_t => t_pwm / 100)
    port map (
    u_i => u_i,
    u_o => u_o_euler_rev);

  i_explicit : explicit port map (
    u_i => u_i,
    u_o => u_o_explicit);

  process
  begin
    wait for t_pwm;
    pay_load <= sin( math_2_pi * f_signal * real( now / 1 ns)*1.0e-9);
    u_i <= a_signal, 0.0 after (0.5 + pay_load * 0.3) * t_pwm;
  end process;

      
  
end architecture rtl_vhdl;
